<?
class MyHttpClient {
    private $baseUrl;
    private $headers;

    public function __construct($baseUrl) {
        $this->baseUrl = $baseUrl;
        $this->headers = [];
    }

    public function setHeader($name, $value) {
        $this->headers[$name] = $value;
    }

    public function get($endpoint) {
        $url = $this->baseUrl . '/' . $endpoint;
        return $this->sendRequest('GET', $url);
    }

    public function post($endpoint, $data) {
        $url = $this->baseUrl . '/' . $endpoint;
        return $this->sendRequest('POST', $url, $data);
    }

    private function sendRequest($method, $url, $data = null) {
        $options = [
            'http' => [
                'method' => $method,
                'header' => $this->buildHeaders(),
                'content' => $data ? http_build_query($data) : null,
            ],
        ];

        $context = stream_context_create($options);
        $response = file_get_contents($url, false, $context);

        // You can add error handling here if needed

        return $response;
    }

    private function buildHeaders() {
        $headers = [];
        foreach ($this->headers as $name => $value) {
            $headers[] = "$name: $value";
        }
        return implode("\r\n", $headers);
    }
}

// Usage
$httpClient = new MyHttpClient('https://api.example.com');

// Add headers (e.g., authentication token)
$httpClient->setHeader('Authorization', 'Bearer your-auth-token-here');

// Example GET request
$getResponse = $httpClient->get('some-endpoint');
echo "GET Response: $getResponse\n";

// Example POST request
$postData = ['key' => 'value'];
$postResponse = $httpClient->post('another-endpoint', $postData);
echo "POST Response: $postResponse\n";
    