<?php
// App etre sur que la persone a le droit defaire
class FrontController {
    private $router;

    public function __construct(Router $router) {
        $this->router = $router;
    }

    public function dispatch(IRequest $request) {
        $match = $this->router->match($request);

        if ($match) {
            $controllerName = $match['controller'];
            $actionName = $match['action'];

            // Utilisez l'injection de dépendances pour créer le contrôleur
            $controller = $this->createController($controllerName);

            if ($controller) {
                // Appeler l'action correspondante
                $controller->$actionName();
            } else {
                // Gérer l'erreur, le contrôleur n'existe pas
                $this->handleError();
            }
        } else {
            // Gérer l'erreur, aucune route correspondante
            $this->handleError();
        }
    }

    private function createController($controllerName) {
        // Utilisez un conteneur d'injection de dépendances pour créer le contrôleur
        return DependencyContainer::create($controllerName);
    }

    private function handleError() {// composant twig
        header("HTTP/1.0 404 Not Found");
        echo "Page not found";
    }
}

?>
<!-- 
Point d'entrée unique : Le front controller est le point d'entrée unique de toutes les requêtes HTTP entrantes dans une application Symfony. Toutes les requêtes sont dirigées vers ce contrôleur, qui est généralement un fichier PHP, tel que public/index.php.

Initialisation : Le front controller initialise l'application en chargeant les dépendances, en configurant l'environnement, en définissant les paramètres globaux, et en préparant l'infrastructure de base nécessaire pour gérer les requêtes HTTP.

Gestion de la requête : Une fois qu'une requête HTTP arrive, le front controller crée un objet Request à partir de cette requête entrante. Il s'occupe également de la gestion de la session, de la gestion des cookies, et de l'analyse des en-têtes HTTP.

Routage vers le contrôleur : Le front controller interagit avec le router pour déterminer quel contrôleur doit être utilisé pour traiter la requête. Il envoie la requête au router pour faire correspondre l'URL demandée à une route et obtenir des informations sur le contrôleur et l'action associés.

Résolution du contrôleur : Une fois que le router a identifié le contrôleur et l'action à appeler, le front controller résout le contrôleur en créant une instance de l'objet contrôleur approprié. Il extrait également l'action à appeler.

Exécution de l'action : Le front controller appelle l'action du contrôleur avec les paramètres nécessaires, généralement en passant l'objet Request. L'action effectue le traitement spécifique à la page, tel que la récupération de données, la manipulation des données, etc.

Génération de la réponse : Après avoir exécuté l'action, le contrôleur génère une réponse HTTP sous la forme d'un objet Response. Cette réponse peut contenir des données à afficher dans le navigateur du client.

Envoi de la réponse : Le front controller envoie finalement la réponse HTTP générée par le contrôleur au client, qui l'affiche dans le navigateur -->


<!-- c'est App -->