<?php
// Chain of Responsibility
// Dans l'app, les requêtes HTTP traversent une série de couches de traitement appelées "middleware". 
// Chaque middleware peut accepter ou refuser la requête. Une fois qu'une requête 
// passe avec succès à travers tous les middleware, elle est finalement traitée par l'application elle-même.
abstract class Middleware implements IHttpMiddleware {
    protected $next;

    public function setNext(IHttpMiddleware $nextMiddleware) {
        $this->next = $nextMiddleware;
    }

    public function handle(IRequest $request, callable $next) {
        if ($this->next !== null) {
            return $this->next->handle($request, $next);
        }
        return $next($request);
    }
}
